from django.apps import apps
from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class CountrywareConfig(AppConfig):
    """
    Configuration entry point for the countryware app
    """
    label = name = 'countryware'
    verbose_name = _("countryware app")



from django.contrib import admin

from .models import Country


@admin.register(Country)
class CountryAdmin(admin.ModelAdmin):
    list_display = ('code', 'name',)
    search_fields = ('code', 'name',)
    ordering = ['code', 'name']



# Generated by Django 2.0.8 on 2018-08-18 11:18

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Country',
            fields=[
                ('code', models.CharField(help_text='Country code', max_length=3, primary_key=True, serialize=False, verbose_name='Code')),
                ('name', models.CharField(blank=True, help_text='Country name (english)', max_length=60, null=True, verbose_name='Name')),
            ],
            options={
                'verbose_name': 'Country',
                'verbose_name_plural': 'Countries',
            },
        ),
    ]



import os
import sys
import codecs
import logging
import json

from django.conf import settings
from django.core.management.base import BaseCommand
from django.core.management.base import CommandError
from django.utils.translation import activate

from ...models import Country
from ...country import get_display
from ... import defaults as defs 

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    # Note: admin:skip
    help = 'Load Country data'
    path = os.path.abspath(os.path.join(os.path.realpath(__file__), '../../../', 'country.json'))

    def add_arguments(self, parser):
        parser.add_argument(
            '-p',
            '--path',
            dest='path',
            default=self.path,
            help='Path to a json country file.'
        )

        parser.add_argument(
            '--flush',
            dest='flush',
            default=False,
            action='store_true',
            help='delete all existing countries in db'
        )

        parser.add_argument(
            '-l',
            '--load',
            dest='load',
            action='store_true',
            default=False,
            help='Load countries from data file'
        )

        parser.add_argument(
            '-o', '--overwrite',
            dest='overwrite',
            action='store_true',
            default=False,
            help='overwrite countries if already found in db'
        )

    def handle(self, *args, **options):
        self.verbosity = options['verbosity']
        path = options['path']
        overwrite = options['overwrite']
        flush = options['flush']
        load = options['load']
        
        if not (flush or load):
            self.print_help("", subcommand='country')
            return
            
        if flush:
            self.flush()

        if load:
            self.load(path, overwrite)

    def flush(self):
        self.stdout.write('You are about to delete all countries from db')
        confirm = input('Are you sure? [yes/no]: ')
        if confirm == 'yes':
            Country.objects.all().delete()
            self.stdout.write('countries deleted from db.')

    def load(self, path, overwrite):
            
        if not os.path.isfile(path):
            self.stdout.write('No country file found at path')
            self.stdout.write(path)
            self.print_help("", subcommand='country')
            return

        activate(defs.DEFAULT_COUNTRY_LANGUAGE_CODE)

        if self.verbosity > 2:
            self.stdout.write('Preparing country file ...')

        fp = codecs.open(path, encoding='utf-8')
        data = json.load(fp)

        new_count, update_count = 0, 0
        for country in data:
            created = False
            defaults = {
                'code': country.get('alpha2Code'),
            }
            if defs.DEFAULT_COUNTRY_LANGUAGE_CODE == 'en':
                defaults['name'] = country.get('name')
            else:
                defaults['name'] = get_display(country.get('alpha2Code'))
            if overwrite:
                instance, created = Country.objects.get_or_create_unique(defaults, ['code'])
            else:
                instance = Country.objects.get_unique_or_none(code=defaults['code'])
                if not instance:
                    instance, created = Country.objects.get_or_create_unique(defaults, ['code'])

            if created:
                new_count += 1
            elif overwrite:
                update_count += 1
        
        self.stdout.write('Created {count} countries'.format(count=new_count))
        self.stdout.write('Updated {count} countries'.format(count=update_count))



from django.core.cache import cache
from django.utils import translation
from django.utils.translation import ugettext as _

_cache_countries = {}
_cache_backend_enabled = None


def check_cache():    
    """ Check if a cache backend is available """
    global _cache_backend_enabled
    if _cache_backend_enabled is None:
        value = 100
        key = 'testing-if-cache-is-enabled'
        try:
            cache.set(key, value)
            if cache.get(key) == value:
                _cache_backend_enabled = True
        except Exception:
            _cache_backend_enabled = False


def get_from_cache(key):
    """ get value for key from cache """
    check_cache()
    if _cache_backend_enabled:
        return cache.get(key)
    return _cache_countries.get(key)


def set_to_cache(key, value):
    """ set value for key to cache """
    check_cache()
    if _cache_backend_enabled:
        cache.set(key, value)
    else:
        _cache_countries[key] = value


def get_cache_key(codes, sorted=False):
    """ get cache key for operation """
    unique = hash(tuple(codes))
    lang = translation.get_language()
    key = '{}-{}-{}'.format(unique, lang, sorted)
    return key



default_app_config = 'countryware.apps.CountrywareConfig'

__author__ = 'Val Neekman @ Neekware Inc. [@vneekman]'
__description__ = "A Django application to translate country names"
__version__ = '0.0.4'




from django.db import models
from django.core.cache import cache
from django.utils.translation import ugettext as _
from django.core.validators import MaxValueValidator, MinValueValidator

from toolware.utils.query import CaseInsensitiveManager, CaseInsensitiveUniqueManager
from .country import get_all_countries_prioritized, get_display
from . import defaults as defs


class Country(models.Model):
    code = models.CharField(
        # Note: admin:skip
        _('Code'),
        max_length=3,
        primary_key=True,
        null=False,
        blank=False,
        # Note: admin:skip
        help_text=_('Country code')
    )

    name = models.CharField(
        # Note: admin:skip
        _('Name'),
        max_length=60,
        null=True,
        blank=True,
        # Note: admin:skip
        help_text=_('Country name (english)'),
    )
  
    # ########## Add new fields above this line #############
    objects = CaseInsensitiveUniqueManager()

    CASE_INSENSITIVE_FIELDS = ['code', 'name']

    @property
    def local_name(self):
        name = get_display(self.code)
        if self.code in name:
            name = self.name
        return name

    def __str__(self):
        return self.local_name

    class Meta:
        # Note: admin:skip
        verbose_name=_('Country')
        # Note: admin:skip
        verbose_name_plural=_('Countries')
